
! :: ----------------------------------------------------------
! :: sumburnedmass
! ::             MASS = sum{ vol(i)*rho(i) }
! ::
! :: INPUTS / OUTPUTS:
! ::  rho        => density field
! ::  rlo,rhi    => index limits of rho aray
! ::  lo,hi      => index limits of grid interior
! ::  dx         => cell size
! ::  mass      <=  total mass
! :: ----------------------------------------------------------
! ::
subroutine ca_sumburnedmass(rho,r_l1,r_l2,r_l3,r_h1,r_h2,r_h3, &
     C12,c_l1,c_l2,c_l3,c_h1,c_h2,c_h3, lo,hi, &
     dx,mass)
  implicit none
  integer           :: r_l1,r_l2,r_l3,r_h1,r_h2,r_h3
  integer           :: c_l1,c_l2,c_l3,c_h1,c_h2,c_h3
  integer           :: lo(3), hi(3)
  double precision  :: mass, dx(3)
  double precision  :: rho(r_l1:r_h1,r_l2:r_h2,r_l3:r_h3)
  double precision  :: C12(c_l1:c_h1,c_l2:c_h2,c_l3:c_h3)

  integer           :: i, j, k
  double precision  :: vol

  vol  = dx(1)*dx(2)*dx(3)
  mass = 0.d0

  !$OMP PARALLEL DO PRIVATE(i,j,k) REDUCTION(+:mass)
  do k = lo(3), hi(3)
     do j = lo(2), hi(2)
        do i = lo(1), hi(1)
           if(C12(i,j,k) .lt. 0.1d0) &
                mass = mass + rho(i,j,k)
        enddo
     enddo
  enddo
  !$OMP END PARALLEL DO

  mass = mass * vol

end subroutine ca_sumburnedmass


! :: ----------------------------------------------------------
! :: sumNi56mass
! ::             MASS = sum{ vol(i)*rho(i) }
! :: 
! :: INPUTS / OUTPUTS:
! ::  rho        => density field
! ::  rlo,rhi    => index limits of rho aray
! ::  lo,hi      => index limits of grid interior
! ::  dx         => cell size
! ::  mass      <=  total mass
! :: ----------------------------------------------------------
! ::

subroutine ca_sumni56mass(rho,r_l1,r_l2,r_l3,r_h1,r_h2,r_h3, &
     ye,ye_l1,ye_l2,ye_l3,ye_h1,ye_h2,ye_h3, lo,hi, &
     dx,mass)
  implicit none
  integer           :: r_l1,r_l2,r_l3,r_h1,r_h2,r_h3
  integer           :: ye_l1,ye_l2,ye_l3,ye_h1,ye_h2,ye_h3
  integer           :: lo(3), hi(3)
  double precision  :: mass, dx(3)
  double precision  :: rho(r_l1:r_h1,r_l2:r_h2,r_l3:r_h3)
  double precision  :: ye(ye_l1:ye_h1,ye_l2:ye_h2,ye_l3:ye_h3)

  integer           :: i, j, k
  double precision  :: vol

  vol  = dx(1)*dx(2)*dx(3)
  mass = 0.d0

  !$OMP PARALLEL DO PRIVATE(i,j,k) REDUCTION(+:mass)
  do k = lo(3), hi(3)
     do j = lo(2), hi(2)
        do i = lo(1), hi(1)
           if(ye(i,j,k) .gt. 0.49d0) &
                mass = mass + rho(i,j,k)
        enddo
     enddo
  enddo
  !$OMP END PARALLEL DO

  mass = mass * vol

end subroutine ca_sumni56mass

! :: ----------------------------------------------------------
! ::
subroutine ca_sumbe(rho,r_l1,r_l2,r_l3,r_h1,r_h2,r_h3, &
     maggrav,mg_l1,mg_l2,mg_l3,mg_h1,mg_h2,mg_h3, &
     lo,hi,problo,dx,be)
  use probdata_module

  implicit none
  integer           :: r_l1,r_l2,r_l3,r_h1,r_h2,r_h3
  integer           :: mg_l1,mg_l2,mg_l3
  integer           :: mg_h1,mg_h2,mg_h3
  integer           :: lo(3), hi(3)
  double precision  :: be, problo(3), dx(3)
  double precision  :: rho(r_l1:r_h1,r_l2:r_h2,r_l3:r_h3)
  double precision  :: maggrav(mg_l1:mg_h1,mg_l2:mg_h2,mg_l3:mg_h3)

  integer           :: i, j, k
  double precision  :: vol
  double precision  :: x,y,z,r

  vol  = dx(1)*dx(2)*dx(3)
  be   = 0.d0

  !$OMP PARALLEL DO PRIVATE(i,j,k,x,y,z,r) REDUCTION(+:be)
  do k = lo(3), hi(3)
     z = problo(3) + (dble(k) + 0.5d0)*dx(3)-center(3)
     do j = lo(2), hi(2)
        y = problo(2) + (dble(j) + 0.5d0)*dx(2)-center(2)
        do i = lo(1), hi(1)
           x = problo(1) + (dble(i) + 0.5d0)*dx(1)-center(1)
           r = sqrt(x*x + y*y + z*z)
           be = be + maggrav(i,j,k)*rho(i,j,k)*r
        enddo
     enddo
  enddo
  !$OMP END PARALLEL DO

  be = be * vol

end subroutine ca_sumbe



! This gets called when castro.restart_with_pert = 1
!
! The goal here is to set up an ignition spot at the finest level
!
subroutine ca_apply_pert(u,u_l1,u_l2,u_l3,u_h1,u_h2,u_h3,lo,hi,xlo,dx)

  use bl_constants_module
  use network, only: nspec, naux
  use eos_module
  use meth_params_module, only: NVAR, URHO, UTEMP, UFS, UFX, UEINT, &
       UMX, UMY, UMZ, UEDEN
  use probdata_module

  implicit none

  integer :: u_l1,u_l2,u_l3,u_h1,u_h2,u_h3
  integer :: lo(3), hi(3)
  double precision :: u(u_l1:u_h1,u_l2:u_h2,u_l3:u_h3,NVAR)
  double precision :: xlo(3), dx(3)

  integer :: i,j,k
  double precision :: x, y, z, dist
  double precision :: rfront, ash_loc, theta, phi, rn
  double precision :: pres, dummy, xn(nspec+naux), ekin
  integer :: ctr, l, m
  double precision :: Ylm

  ! this code was ripped from what used to be in the initdata routine
  do k = lo(3), hi(3)
     z = xlo(3) + (dble(k-lo(3)) + 0.5d0) * dx(3) - ash_center(3)
     do j = lo(2), hi(2)
        y = xlo(2) + (dble(j-lo(2)) + 0.5d0) * dx(2) - ash_center(2)
        do i = lo(1), hi(1)
           x = xlo(1) + (dble(i-lo(1)) + 0.5d0) * dx(1) - ash_center(1)

           dist = sqrt(x*x + y*y + z*z)
           rfront = 0.d0

           if (pertmag .gt. 0.d0) then
              ctr = 0
              do l = lmodemin, lmodemax
                 do m = 0, l
                    ctr = ctr + 1

                    if (dist .le. dx(1)) then
                       theta = 0.d0
                       phi = 0.d0
                    else
                       theta = atan2(sqrt(x*x+y*y),z)-M_PI
                       phi = atan2(y,x)
                    endif
                    rfront = rfront + betalm(l,m) + Ylm(l,m,phi,cos(theta))
                 enddo
              enddo
              if (ctr .gt. 0) rfront = rfront/ctr
           endif

           ash_loc = ash_radius*(1.d0+pertmag*rfront)

           if (dist .lt. 2.d0*ash_loc) then

              ! primitive forms
              u(i,j,k,UMX:UMZ        ) = u(i,j,k,UMX:UMZ)/u(i,j,k,URHO)
              u(i,j,k,UEINT          ) = u(i,j,k,UEINT)/u(i,j,k,URHO)
              u(i,j,k,UFS:UFS+nspec-1) = u(i,j,k,UFS:UFS+nspec-1)/u(i,j,k,URHO)
              u(i,j,k,UFX:UFX+naux-1 ) = u(i,j,k,UFX:UFX+naux-1)/u(i,j,k,URHO)

              if (include_random .eq. 1) then
                 call blutilrand(rn)

                 pt_index_eos(1) = i
                 pt_index_eos(2) = j
                 pt_index_eos(3) = k

                 if ((rn .gt. 0.5d0 .and. dist .lt. ash_loc) .or. &
                      (iextend_pert .eq. 1 .and. rn .lt. 0.3d0 .and. dist .ge. ash_loc)) then

                    xn(1      :nspec     ) = u(i,j,k,UFS:UFS+nspec-1)
                    xn(nspec+1:nspec+naux) = u(i,j,k,UFX:UFX+naux -1)

                    ! get the pressure so that we can perturb at 
                    ! constant pressure
                    call eos_given_RTX(u(i,j,k,UEINT),pres,&
                         u(i,j,k,URHO),u(i,j,k,UTEMP),xn,pt_index_eos)

                    ! switch to ignited material as perturbation
                    u(i,j,k,UTEMP) = 8.5d9
                    u(i,j,k,URHO ) = 2.15d9   ! just a guesses from table
                    u(i,j,k,UFS  ) = 1.d-5    ! this seems inconsistent....
                    u(i,j,k,UFX+1) = 8.166d0  ! BE/A
                    u(i,j,k,UFX+2) = 11.6d0   ! Abar

                    xn(1      :nspec     ) = u(i,j,k,UFS:UFS+nspec-1)
                    xn(nspec+1:nspec+naux) = u(i,j,k,UFX:UFX+naux -1)

                    ! get updated thermodynamics
                    call eos_given_TPX(u(i,j,k,UEINT),pres,u(i,j,k,URHO),&
                         u(i,j,k,UTEMP),xn,pt_index_eos)
                 endif
              else
                 if (dist .lt. ash_loc) then

                    xn(1      :nspec     ) = u(i,j,k,UFS:UFS+nspec-1)
                    xn(nspec+1:nspec+naux) = u(i,j,k,UFX:UFX+naux -1)

                    ! get the pressure so that we can perturb at 
                    ! constant pressure
                    call eos_given_RTX(u(i,j,k,UEINT),pres,&
                         u(i,j,k,URHO),u(i,j,k,UTEMP),xn,pt_index_eos)

                    ! switch to ignited material
                    ! switch to ignited material
                    u(i,j,k,UTEMP) = 8.5d9
                    u(i,j,k,URHO ) = 2.15d9   ! just a guesses from table
                    u(i,j,k,UFS  ) = 1.d-5    ! this seems inconsistent....
                    u(i,j,k,UFX+1) = 8.166d0  ! BE/A
                    u(i,j,k,UFX+2) = 11.6d0   ! Abar

                    xn(1      :nspec     ) = u(i,j,k,UFS:UFS+nspec-1)
                    xn(nspec+1:nspec+naux) = u(i,j,k,UFX:UFX+naux -1)

                    ! get updated thermodynamics
                    call eos_given_TPX(u(i,j,k,UEINT),pres,u(i,j,k,URHO),&
                         u(i,j,k,UTEMP),xn,pt_index_eos)

                 endif
              endif

              ! convert back to conservative quantities
              ekin = 0.5*u(i,j,k,URHO)*&
                   (u(i,j,k,UMX)**2+u(i,j,k,UMY)**2+u(i,j,k,UMZ)**2)
              u(i,j,k,UMX:UMZ        ) = u(i,j,k,UMX:UMZ)*u(i,j,k,URHO)
              u(i,j,k,UEINT          ) = u(i,j,k,UEINT)*u(i,j,k,URHO)
              u(i,j,k,UEDEN          ) = u(i,j,k,UEINT) + ekin
              u(i,j,k,UFS:UFS+nspec-1) = u(i,j,k,UFS:UFS+nspec-1)*u(i,j,k,URHO)
              u(i,j,k,UFX:UFX+naux -1) = u(i,j,k,UFX:UFX+naux -1)*u(i,j,k,URHO)
           endif
        enddo
     enddo
  enddo

end subroutine ca_apply_pert


!
! spherical harmonics helper functions
!

double precision function Ylm(l,m,phi,costheta)

  use bl_constants_module

  double precision :: phi, costheta, facN, facD, plgndr
  integer :: m, l, n

  facN = l-m
  if (facN .eq. 0) facN=1
  do n = l-m-1, 2, -1
     facN = facN*n
  enddo
  facD = l+m
  if (facD .eq. 0) facD=1
  do n = l+m-1,2,-1
     facD = facD*n
  enddo
  Ylm = sqrt((2*l+1)*facN/(4*M_PI*facD))*plgndr(l,m,costheta)*cos(m*phi)
end function Ylm

double precision function plgndr(l,m,x)

  integer :: l,m
  double precision :: x
  integer :: i, ll
  double precision :: fact, pll, pmm, pmmp1, somx2

  if(m.lt.0.or.m.gt.l.or.abs(x).gt.1.) then
     call bl_abort('bad arguments in plgndr')
  endif
  pmm=1.d0
  if(m.gt.0) then
     somx2=sqrt((1.d0-x)*(1.d0+x))
     fact=1.d0
     do i=1,m
        pmm=-pmm*fact*somx2
        fact=fact+2.d0
     enddo
  endif
  if(l.eq.m) then
     plgndr=pmm
  else
     pmmp1=x*(2*m+1)*pmm
     if(l.eq.m+1) then
        plgndr=pmmp1
     else
        do ll=m+2,l
           pll=(x*(2*ll-1)*pmmp1-(ll+m-1)*pmm)/(ll-m)
           pmm=pmmp1
           pmmp1=pll
        enddo
        plgndr=pll
     endif
  endif

end function plgndr



subroutine ca_ashsum(rho,r_l1,r_l2,r_l3,r_h1,r_h2,r_h3,c12,c_l1,c_l2,c_l3,c_h1,c_h2,c_h3,&
     lo,hi,prob_lo,dx,mass,idir)

  use probdata_module, only: center

  implicit none

  integer :: r_l1,r_l2,r_l3,r_h1,r_h2,r_h3,c_l1,c_l2,c_l3,c_h1,c_h2,c_h3
  double precision :: rho(r_l1:r_h1,r_l2:r_h2,r_l3:r_h3)
  double precision :: c12(c_l1:c_h1,c_l2:c_h2,c_l3:c_h3)
  integer :: lo(3), hi(3), idir
  double precision :: prob_lo(3), dx(3)
  double precision :: mass

  integer :: i, j, k
  double precision :: x, y, z, vol

  vol = dx(1)*dx(2)*dx(3)
  mass = 0.0d0

  if (idir .eq. 0) then
     !$OMP PARALLEL DO PRIVATE(i,j,k,x) REDUCTION(+:mass)
     do i = lo(1), hi(1)
        x = prob_lo(1) + (dble(i)+0.5d0) * dx(1) - center(1)
        do k = lo(3), hi(3)
           do j = lo(2), hi(2)
              if (c12(i,j,k) < 0.1d0) then
                 mass = mass + rho(i,j,k) * x
              endif
           enddo
        enddo
     enddo
     !$OMP END PARALLEL DO
  else if (idir .eq. 1) then
     !$OMP PARALLEL DO PRIVATE(i,j,k,y) REDUCTION(+:mass)
     do j = lo(2), hi(2)
        y = prob_lo(2) + (dble(j)+0.5d0) * dx(2) - center(2)
        do k = lo(3), hi(3)
           do i = lo(1), hi(1)
              if (c12(i,j,k) < 0.1d0) then
                 mass = mass + rho(i,j,k) * y
              endif
           enddo
        enddo
     enddo
     !$OMP END PARALLEL DO
  else
     !$OMP PARALLEL DO PRIVATE(i,j,k,z) REDUCTION(+:mass)
     do k = lo(3), hi(3)
        z = prob_lo(3) + (dble(k)+0.5d0) * dx(3) - center(3)
        do j = lo(2), hi(2)
           do i = lo(1), hi(1)
              if (c12(i,j,k) < 0.1d0) then
                 mass = mass + rho(i,j,k) * z
              endif
           enddo
        enddo
     enddo
     !$OMP END PARALLEL DO
  end if


  mass = mass * vol

end subroutine ca_ashsum
