
      subroutine PROBINIT (init,name,namlen,problo,probhi)

      use probdata_module
      use network, only : network_init,nspec,naux
      use nse_module, only : nsetable_init
      use burning_module
      implicit none 

      integer :: init,namlen,untin,i,k
      integer :: name(namlen)

      double precision problo(3), probhi(3)

      namelist /fortin/ &
           model_name,  &
           denerr0, denerr1, denerr2, denerr3, &
           dengrad, denthreshold,  max_denerr_lev, max_dengrad_lev, &
           temperr, tempgrad, max_temperr_lev, max_tempgrad_lev, &
           specerr, specgrad, max_specerr_lev, max_specgrad_lev, &
           velerr, velgrad, max_velerr_lev, max_velgrad_lev, &
           presserr, pressgrad, max_presserr_lev, max_pressgrad_lev,&
           raderr, radgrad, max_raderr_lev, max_radgrad_lev, &
           flame_thickening_factor, flamevel, taudiv, bea_fuel, abar_fuel, &
           ash_x, ash_y, ash_z, ash_radius, pertmag, include_random, iextend_pert, &
           zero_velocity_field
!
!     Build "probin" filename -- the name of file containing fortin namelist.
!     
      integer   :: ipos,maxlen
      parameter (maxlen=127)
      character probin*(maxlen)
      character (len=256) :: header_line

      integer :: nvars_model_file
      integer :: j

      integer :: myproc

      call network_init()
      call nsetable_init()
      call burningtable_init()

      nvars_model_file = 6  

      if (namlen .gt. maxlen) then
         write(6,*) 'probin file name too long'
         stop
      end if

      do i = 1, namlen
         probin(i:i) = char(name(i))
      end do

! set namelist defaults

      bea_fuel = 7.828d0
!      abar_fuel= 13.7d0
     
      ! this is what helmeos thinks 50% C/O should be
      abar_fuel = 13.714285714285715d0

      ! Flame parameters -- these must be set in the probin file
      ash_radius              = -1.d0
      flame_thickening_factor = -1   
      flamevel                = -1.d0
      taudiv                  =  1.d0

!     pertubation defaults to original random numbers
      pertmag = 0.d0
      include_random = 1
      iextend_pert = 1

      denerr0 = 1.d20
      denerr1 = 1.d20
      denerr2 = 1.d20
      denerr3 = 1.d20
      dengrad = 1.d20
      denthreshold = 1.d20
      max_denerr_lev = -1
      max_dengrad_lev = -1

      presserr = 1.d20
      pressgrad = 1.d20
      max_presserr_lev = -1
      max_pressgrad_lev = -1

      velgrad = 1.d20
      max_velgrad_lev = -1

      ash_x = 0.d0
      ash_y = 0.d0
      ash_z = 0.d0

      zero_velocity_field = .false.

!     Read namelists in probin file
      untin = 9
      open(untin,file=probin(1:namlen),form='formatted',status='old')
      read(untin,fortin)
      close(unit=untin)

      ! Make sure these are included in the probin file.
      if (ash_radius .lt. 0.d0) then
         call bl_error("ERROR in PROBINIT: must initialize       ash_radius in probin")
      end if
      if (flame_thickening_factor .lt. 0) then
         call bl_error("ERROR in PROBINIT: must initialize flame_thickening_factor in probin")
      end if
      if (flamevel .lt. 0.d0) then
         call bl_error("ERROR in PROBINIT: must initialize         flamevel in probin")
      end if

!     Open file of initial profile
      open(unit=99,file=model_name)

      ! the model file is assumed to be of the follow form:
      ! # npts = 896
      ! # num of variables = 6
      ! # density
      ! # temperature
      ! # pressure
      ! # carbon-12
      ! # oxygen-16
      ! # magnesium-24
      ! 195312.5000  5437711139.  8805500.952   .4695704813E+28  0.3  0.7  0
      ! 585937.5000  5410152416.  8816689.836  0.4663923963E+28  0.3  0.7  0

      ! the first line has the number of points in the model
      read (99, '(a256)') header_line
      ipos = index(header_line, '=') + 1
      read (header_line(ipos:),*) npts_model

      ! now read in the number of variables
      read (99, '(a256)') header_line
      ipos = index(header_line, '=') + 1
      read (header_line(ipos:),*) nvars_model_file

      ! now read in the names of the variables
      do i = 1, nvars_model_file
         read (99, '(a256)') header_line
      enddo

      allocate(hse_r(npts_model),hse_rho(npts_model),hse_t(npts_model),hse_p(npts_model))
      allocate(hse_s(3,npts_model))

      do k = 1,npts_model
         read(99,*) hse_r(k), hse_rho(k), hse_t(k), hse_p(k), &
                    hse_s(1,k), hse_s(2,k), hse_s(3,k)
      end do
      close(99)

      center(1) = problo(1) + 0.5d0 * (probhi(1)-problo(1))
      center(2) = problo(2) + 0.5d0 * (probhi(2)-problo(2))
      center(3) = problo(3) + 0.5d0 * (probhi(3)-problo(3))

      ash_center(1) = ash_x
      ash_center(2) = ash_y
      ash_center(3) = ash_z

!       ash_center(1) = center(1) + offcenter_radius/sqrt(3.d0)
!       ash_center(2) = center(2) + offcenter_radius/sqrt(3.d0)
!       ash_center(3) = center(3) + offcenter_radius/sqrt(3.d0)

      ! read in f_width and f_vel
      open(unit=90,file="fvalue")
      do i = 1, 17
          read(90,*) flogrho(i), f_width(i), f_vel(i)
      enddo

      ! read in minimum flame velocity properties (laminar) for
      ! interpolation, to ensure we don't specify something too slow
      ! 
      ! Minumum values are from v_cond in Table 3 of Timmes & Woosley
      ! (1992)
      do i = 1, 11
         read(90,*) l_logrho(i), l_vel(i)
         l_vel(i) = l_vel(i)*1.d5         ! swap from km / s to cm / s
      enddo
      close(90)

!   initialize random numbers for spherical harmonics

      call blutilinitrand(111397)

      do i = lmodemin,lmodemax
         do j = 0,i
            call blutilrand(alphalm(i,j))
            call blutilrand(betalm(i,j))
            call blutilrand(gammalm(i,j))
         end do
      end do


!     Init BoxLib random number generator so each processor has a different seed
      call bl_pd_myproc(myproc)
      call blutilinitrand(myproc)

      end subroutine PROBINIT


! ::: -----------------------------------------------------------
! ::: This routine is called at problem setup time and is used
! ::: to initialize data on each grid.  
! ::: 
! ::: NOTE:  all arrays have one cell of ghost zones surrounding
! :::        the grid interior.  Values in these cells need not
! :::        be set here.
! ::: 
! ::: INPUTS/OUTPUTS:
! ::: 
! ::: level     => amr level of grid
! ::: time      => time at which to init data             
! ::: lo,hi     => index limits of grid interior (cell centered)
! ::: nstate    => number of state components.  You should know
! :::		   this already!
! ::: state     <=  Scalar array
! ::: delta     => cell size
! ::: xlo,xhi   => physical locations of lower left and upper
! :::              right hand corner of grid.  (does not include
! :::		   ghost region).
! ::: -----------------------------------------------------------
!
!  NOTE: THIS ROUTINE WILL NEVER BE CALLED IF WE ARE INIITIALIZING
!        FROM A MAESTRO RUN
!
      subroutine ca_initdata(level,time,lo,hi,nscal, &
                            state,state_l1,state_l2,state_l3,&
                            state_h1,state_h2,state_h3,delta,xlo,xhi)

      use bl_constants_module
      use probdata_module
      use eos_module
      use network, only : nspec,naux
      use interpolate_module
      use meth_params_module, only : NVAR, URHO, UMX, UMY, UMZ,UTEMP, UEDEN, UEINT, UFS, UFX

      implicit none
 
      integer          :: level, nscal
      integer          :: lo(3), hi(3)
      integer          :: state_l1,state_l2,state_l3
      integer          :: state_h1,state_h2,state_h3
      double precision :: state(state_l1:state_h1,state_l2:state_h2,state_l3:state_h3,NVAR)
      double precision :: time, delta(3)
      double precision :: xlo(3), xhi(3)

      double precision :: x,y,z,dist,pres
      integer          :: i,j,k,n
      integer          :: l,m,ctr
      integer          :: pt_index(3)
      double precision :: xtemp,xloc,yloc,zloc,phi,theta,ash_loc,rfront
      double precision :: rn
      double precision :: Ylm
      double precision :: xn(nspec+naux)

      ! Interpolate rho, T and X
      do k = lo(3), hi(3)
         z = xlo(3) + (dble(k-lo(3)) + 0.5d0) * delta(3) - center(3)
      do j = lo(2), hi(2)
         y = xlo(2) + (dble(j-lo(2)) + 0.5d0) * delta(2) - center(2)
      do i = lo(1), hi(1)
         x = xlo(1) + (dble(i-lo(1)) + 0.5d0) * delta(1) - center(1)

         dist = sqrt(x*x + y*y + z*z)
 
         state(i,j,k,URHO ) = interpolate(dist,npts_model,hse_r,hse_rho)
         state(i,j,k,UTEMP) = interpolate(dist,npts_model,hse_r,hse_t)

         ! C O He Ne Mg Si-Ca Fe Ye BeA Abar
         state(i,j,k,UFS) = 0.5d0
         state(i,j,k,UFS+1) = 0.5d0
         do n= 3, 7
            state(i,j,k,UFS+n-1) = 0.d0
         enddo

         state(i,j,k,UFX  )   = 0.5d0
         state(i,j,k,UFX+1) = bea_fuel
         state(i,j,k,UFX+2) = abar_fuel

         pt_index(1) = i
         pt_index(2) = j
         pt_index(3) = k

         ! stack species and auxiliary variables into one vector
         xn(1      :nspec     ) = state(i,j,k,UFS:UFS+nspec-1)
         xn(nspec+1:nspec+naux) = state(i,j,k,UFX:UFX+naux -1)

         ! Compute energy from rho,T and X
         call eos_given_RTX(state(i,j,k,UEINT),pres,state(i,j,k,URHO),state(i,j,k,UTEMP), &
                            xn,pt_index)

      enddo
      enddo
      enddo

         ! set up hot ash spots around ash center

      do k = lo(3), hi(3)
         z = xlo(3) + (dble(k-lo(3)) + 0.5d0) * delta(3) - ash_center(3)
      do j = lo(2), hi(2)
         y = xlo(2) + (dble(j-lo(2)) + 0.5d0) * delta(2) - ash_center(2)
      do i = lo(1), hi(1)
         x = xlo(1) + (dble(i-lo(1)) + 0.5d0) * delta(1) - ash_center(1)

         dist = sqrt(x*x + y*y + z*z)
         rfront = 0.d0
         if(pertmag.gt.0.d0)then
           ctr = 0
           do l = lmodemin,lmodemax
              do m = 0,l
                 ctr = ctr + 1

! set local coordinates
                 if (dist.LE.delta(1)) then
                    theta = zero
                    phi = zero
                 else
!                     xtemp = cos(2.d0*M_PI*alphalm(l,m))*x + sin(2.d0*M_PI*alphalm(l,m))*y
!                     yloc = -sin(2.d0*M_PI*alphalm(l,m))*x + cos(2.d0*M_PI*alphalm(l,m))*y
  
!                     xloc = cos(2.d0*M_PI*gammalm(l,m))*xtemp + sin(2.d0*M_PI*gammalm(l,m))*z
!                     zloc = -sin(2.d0*M_PI*gammalm(l,m))*xtemp + cos(2.d0*M_PI*gammalm(l,m))*z
  
!                     theta = ATAN2(SQRT((xloc-ash_center(1))**2+(yloc-ash_center(2))**2)/dist,(zloc-ash_center(3))/dist)-M_PI
!                     phi = ATAN2((yloc-ash_center(2))/dist,(xloc-ash_center(1))/dist)

                    theta = atan2(sqrt(x*x+y*y),z)-M_PI
                    phi = atan2(y,x)
                 endif
                 rfront = rfront + betalm(l,m) * Ylm(l,m,phi,COS(theta))
  
              end do
            end do
            if (ctr.GT.0) rfront = rfront/ctr
          endif
          ash_loc = ash_radius*(1.d0+pertmag*rfront)


          if (dist .lt. 2.d0*ash_loc) then

             if(include_random .eq. 1)then
                call blutilrand(rn)

                pt_index(1) = i
                pt_index(2) = j
                pt_index(3) = k
                
                if ((rn .gt. 0.5d0 .and. dist .lt. ash_loc ).or. &
                   (iextend_pert.eq.1 .and. rn .lt. 0.3d0 .and. dist .ge. ash_loc )) then

                   ! stack species and auxiliary variables into one vector
                   xn(1      :nspec     ) = state(i,j,k,UFS:UFS+nspec-1)
                   xn(nspec+1:nspec+naux) = state(i,j,k,UFX:UFX+naux -1)
                   
                   ! get the hpresure so that we can perturb at 
                   ! constant pressure
                   call eos_given_RTX(state(i,j,k,UEINT),pres,state(i,j,k,URHO),state(i,j,k,UTEMP), &
                                      xn,pt_index)

                   ! switch to ignited material
                   state(i,j,k,UTEMP)= 8.5d9


                   ! give a guess of rho from the table
                   state(i,j,k,URHO) = 2.15d9

                   ! set the species from the table; the EOS and
                   ! burner only care about the carbon but the
                   ! check_initial_species routine cares about
                   ! sum(X_i)
                   state(i,j,k,UFS  )= 1.5d-5     ! carbon; we set this the 
                                                  ! same as in apply_pert
                   state(i,j,k,UFS+1)= 3.442d-4   ! oxygen
                   state(i,j,k,UFS+2)= 2.005d-1   ! helium
                   state(i,j,k,UFS+3)= 2.486d-5   ! neon
                   state(i,j,k,UFS+4)= 6.485d-4   ! magnesium
                   state(i,j,k,UFS+5)= 1.033d-1   ! silicon
                   state(i,j,k,UFS+6) = 1.d0 - &
                        sum(state(i,j,k,UFS:UFS+nspec-2)) ! rest in iron

                   state(i,j,k,UFX+1)= 8.166d0
                   state(i,j,k,UFX+2)= 11.6d0 

                   ! stack species and auxiliary variables into one vector
                   xn(1      :nspec     ) = state(i,j,k,UFS:UFS+nspec-1)
                   xn(nspec+1:nspec+naux) = state(i,j,k,UFX:UFX+naux -1)

                   !to keep pressure unchanged
                   call eos_given_TPX(state(i,j,k,UEINT),pres,state(i,j,k,URHO),state(i,j,k,UTEMP), &
                                      xn,pt_index)

                endif
             else
                if (dist .lt. ash_loc) then

                   ! stack species and auxiliary variables into one vector
                   xn(1      :nspec     ) = state(i,j,k,UFS:UFS+nspec-1)
                   xn(nspec+1:nspec+naux) = state(i,j,k,UFX:UFX+naux -1)

                   call eos_given_RTX(state(i,j,k,UEINT),pres,state(i,j,k,URHO),state(i,j,k,UTEMP), &
                                      xn,pt_index)


                   ! switch to ignited material
                   state(i,j,k,UTEMP)= 8.5d9
  
                   !give a guess of rho from the table
                   state(i,j,k,URHO) = 2.15d9

                   ! set the species from the table; the EOS and
                   ! burner only care about the carbon but the
                   ! check_initial_species routine cares about
                   ! sum(X_i)
                   state(i,j,k,UFS  )= 1.5d-5     ! carbon; we set this the 
                                                  ! same as in apply_pert
                   state(i,j,k,UFS+1)= 3.442d-4   ! oxygen
                   state(i,j,k,UFS+2)= 2.005d-1   ! helium
                   state(i,j,k,UFS+3)= 2.486d-5   ! neon
                   state(i,j,k,UFS+4)= 6.485d-4   ! magnesium
                   state(i,j,k,UFS+5)= 1.033d-1   ! silicon
                   state(i,j,k,UFS+6) = 1.d0 - &
                        sum(state(i,j,k,UFS:UFS+nspec-2)) ! rest in iron

                   state(i,j,k,UFX+1)= 8.166d0
                   state(i,j,k,UFX+2)= 11.6d0 

                   ! stack species and auxiliary variables into one vector
                   xn(1      :nspec     ) = state(i,j,k,UFS:UFS+nspec-1)
                   xn(nspec+1:nspec+naux) = state(i,j,k,UFX:UFX+naux -1)

                   !to keep pressure unchanged
                   call eos_given_TPX(state(i,j,k,UEINT),pres,state(i,j,k,URHO),state(i,j,k,UTEMP), &
                                      xn,pt_index)

                endif
              endif
         endif

      enddo
      enddo
      enddo

      ! switch to conserved quantities
      do k = lo(3), hi(3)
      do j = lo(2), hi(2)
      do i = lo(1), hi(1)
        state(i,j,k,UMX  ) = 0.d0
        state(i,j,k,UMY  ) = 0.d0
        state(i,j,k,UMZ  ) = 0.d0
        state(i,j,k,UEDEN) = state(i,j,k,URHO) * state(i,j,k,UEINT)
        state(i,j,k,UEINT) = state(i,j,k,URHO) * state(i,j,k,UEINT)
        state(i,j,k,UFS:UFS+nspec-1) = state(i,j,k,URHO) * state(i,j,k,UFS:UFS+nspec-1)
        state(i,j,k,UFX:UFX+ naux-1) = state(i,j,k,URHO) * state(i,j,k,UFX:UFX+ naux-1)
      enddo
      enddo
      enddo

      end subroutine ca_initdata


   subroutine ca_initdata_maestro(lo,hi,MAESTRO_init_type, &
                                  state,state_l1,state_l2,state_l3, &
                                  state_h1,state_h2,state_h3, &
                                  dx,dr,xlo,xhi,p0,MAESTRO_npts_model,level)
     use probdata_module
     use interpolate_module
     use eos_module
     use meth_params_module, only : NVAR, URHO, UMX, UMY, UMZ, UEDEN, UEINT, UFS, UTEMP, UFX
     use network, only: nspec, naux

     implicit none

     integer lo(3), hi(3), MAESTRO_init_type, level
     integer MAESTRO_npts_model
     integer state_l1,state_l2,state_l3,state_h1,state_h2,state_h3
     double precision xlo(3), xhi(3), dx(3), dr
     double precision p0(0:MAESTRO_npts_model-1)
     double precision state(state_l1:state_h1,state_l2:state_h2,state_l3:state_h3,NVAR)

     ! local variables
     double precision ekin,xcen,ycen,zcen,radius,dnew,tnew
     double precision pressure,entropy,minpres
     double precision :: xn(nspec+naux)

     integer i,j,k,n,index

     ! Recall:
     ! UFX     : Ye
     ! UFX + 1 : BE / A
     ! UFX + 2 : Abar

     minpres = p0(MAESTRO_npts_model-1)

     state(:,:,:,UFS:UFS+nspec+naux-1) = 0.0d0

     ! hard-code species and auxiliary variables (not mass weighted yet)
     do k=lo(3), hi(3)
        do j = lo(2), hi(2)
           do i = lo(1), hi(1)

              ! reset the composition to be 50/50 carbon/oxygen for simplicity
              state(i,j,k,UFS) = 0.5d0
              state(i,j,k,UFS+1) = 0.5d0

              state(i,j,k,UFX) = 0.5d0
              state(i,j,k,UFX+1) = bea_fuel
              state(i,j,k,UFX+2) = abar_fuel

              if (zero_velocity_field) state(i,j,k,UMX:UMZ) = 0.0d0

!               ! this is sum(X_i/A_i) = sum(Y_i)
!               dnew = state(i,j,k,UFS  ) / 12.d0 + &
!                      state(i,j,k,UFS+1) / 16.d0 + &
!                      state(i,j,k,UFS+2) / 18.d0

!               ! this is sum(Z_i * Y_i)
!               tnew = 6.d0  * state(i,j,k,UFS  ) / 12.d0 + &
!                      8.d0  * state(i,j,k,UFS+1) / 16.d0 + &
!                      8.8d0 * state(i,j,k,UFS+2) / 18.d0

!               ! this is based on the wdconvect ignition_chamulak network
!               state(i,j,k,UFX  ) = tnew

!               ! this is not quite right; should be based on incoming composition
!               state(i,j,k,UFX+1) = bea_fuel

!               ! this is based on the wdconvect ignition_chamulak network
!               state(i,j,k,UFX+2) = 1.d0/dnew


           end do
        end do
     end do

     ! compute p0 and add pi if necessary
     do k = lo(3), hi(3)
        zcen = xlo(3) + dx(3)*(float(k-lo(3)) + 0.5d0) - center(3)
        do j = lo(2), hi(2)
           ycen = xlo(2) + dx(2)*(float(j-lo(2)) + 0.5d0) - center(2)
           do i = lo(1), hi(1)
              xcen = xlo(1) + dx(1)*(float(i-lo(1)) + 0.5d0) - center(1)
              
              radius = sqrt(xcen**2 + ycen**2 + zcen**2)
              index = int(radius/dr)
              
              if (index .eq. 0) then
                 index = 1
              else if (index .ge. MAESTRO_npts_model-1) then
                 index = MAESTRO_npts_model-2
              end if
              
              call quad_interp(radius, &
                               (index-0.5d0)*dr,(index+0.5d0)*dr,(index+1.5d0)*dr, &
                               pressure, &
                               p0(index-1),p0(index),p0(index+1))
              
              if (MAESTRO_init_type .eq. 1) then
                 ! set pressure = p0
                 state(i,j,k,UEDEN) = pressure
              else
                 ! set pressure = p0+pi
                 state(i,j,k,UEDEN) = state(i,j,k,UEDEN) + pressure
              end if

           end do
        end do
     end do

     if (MAESTRO_init_type .eq. 1 .or. MAESTRO_init_type .eq. 2) then

        do k=lo(3), hi(3)
           do j = lo(2), hi(2)
              do i = lo(1), hi(1)

                 ! load pressure from our temporary storage field
                 pressure = max(state(i,j,k,UEDEN),minpres)

                 ! stack species and auxiliary variables into one vector
                 xn(1      :nspec     ) = state(i,j,k,UFS:UFS+nspec-1)
                 xn(nspec+1:nspec+naux) = state(i,j,k,UFX:UFX+naux -1)

                 ! compute e and T
                 call eos_e_given_RPX(state(i,j,k,UEINT),state(i,j,k,UTEMP), &
                                      state(i,j,k,URHO),pressure,xn)

                 ! compute kinetic energy
                 ekin = 0.5*state(i,j,k,URHO)* &
                      (state(i,j,k,UMX)**2+state(i,j,k,UMY)**2+state(i,j,k,UMZ)**2)

                 ! convert velocity to momentum
                 state(i,j,k,UMX:UMZ) = state(i,j,k,UMX:UMZ)*state(i,j,k,URHO)

                 ! compute rho*e
                 state(i,j,k,UEINT) = state(i,j,k,URHO) * state(i,j,k,UEINT)

                 ! compute rho*E = rho*e + ke
                 state(i,j,k,UEDEN) = state(i,j,k,UEINT) + ekin

                 ! convert X to rhoX for species and auxiliary variables
                 do n = 1,nspec
                    state(i,j,k,UFS+n-1) = state(i,j,k,URHO) * state(i,j,k,UFS+n-1)
                 end do
                 do n = 1,naux
                    state(i,j,k,UFX+n-1) = state(i,j,k,URHO) * state(i,j,k,UFX+n-1)
                 end do

              end do
           end do
        end do


     else if (MAESTRO_init_type .eq. 3) then

        call bl_error("MAESTRO_init_type = 3 not defined in Prob_3d.f90")

     else if (MAESTRO_init_type .eq. 4) then

        call bl_error("MAESTRO_init_type = 4 not defined in Prob_3d.f90")

     end if

   end subroutine ca_initdata_maestro

   ! ::: -----------------------------------------------------------

   subroutine ca_initdata_makemodel(model,model_size,MAESTRO_npts_model, &
                                    rho0,tempbar,dx,dr,r_model_start)

     use network, only: nspec, naux
     use eos_module
     use bl_constants_module, only: M_PI, FOUR3RD, HALF
     use fundamental_constants_module, only: Gconst
     use probdata_module, only: bea_fuel, abar_fuel

     implicit none

     integer model_size,MAESTRO_npts_model
     double precision model(model_size,0:MAESTRO_npts_model-1)
     double precision rho0   (0:MAESTRO_npts_model-1)
     double precision tempbar(0:MAESTRO_npts_model-1)
     double precision dx(3), dr
     integer r_model_start

     ! local
     double precision xznl      (0:MAESTRO_npts_model-1)
     double precision xznr      (0:MAESTRO_npts_model-1)
     double precision xzn_hse   (0:MAESTRO_npts_model-1)
     double precision M_enclosed(0:MAESTRO_npts_model-1)
     double precision pres      (0:MAESTRO_npts_model-1)

     double precision g_zone,temp_zone,dens_zone,pres_zone,xn(nspec+naux)
     double precision p_want,dpd,drho
     double precision low_density_cutoff,temp_fluff,temp_fluff_density_cutoff
     double precision TOL

     integer MAX_ITER
     integer i,n,iter

     logical converged_hse,hit_low_density_cutoff

     MAX_ITER = 250
     TOL = 1.e-10

     low_density_cutoff = 1.d-4
     temp_fluff_density_cutoff = 1.d3

     temp_fluff = 5.d6

     hit_low_density_cutoff = .false.

     ! compute physical coordinates of left-edge, right-edge, and bin-center
     do i=0,MAESTRO_npts_model-1
        xznl(i) = (dble(i))*dr
        xznr(i) = (dble(i)+1.d0)*dr
        xzn_hse(i) = 0.5d0*(xznl(i) + xznr(i))
     end do

     ! compute M_enclosed
     M_enclosed(0) = FOUR3RD*M_PI*dr**3*rho0(0)
     
     ! M = M + (4/3)*PI*(r_outer^3 - r_inner^3)*rho
     do i=1,MAESTRO_npts_model-1
        M_enclosed(i) = M_enclosed(i-1) + &
             FOUR3RD*M_PI*(xznr(i) - xznl(i)) * &
             (xznr(i)**2 + xznl(i)*xznr(i) + xznl(i)**2)*rho0(i)
     end do

     ! hard-code species and auxiliary quantities
     xn(1) = 0.5d0
     xn(2) = 0.5d0
     xn(3) = 0.d0
     xn(4) = 0.d0
     xn(5) = 0.d0
     xn(6) = 0.d0
     xn(7) = 0.d0
     xn(8) = 0.5d0
     xn(9) = bea_fuel
     xn(10)= abar_fuel

     ! compute pressure using the EOS
     den_eos  = rho0(r_model_start)
     temp_eos = tempbar(r_model_start)
     xn_eos(:) = xn(:)

     call eos(eos_input_rt, den_eos, temp_eos, &
              xn_eos, &
              p_eos, h_eos, e_eos, &
              cv_eos, cp_eos, xne_eos, eta_eos, pele_eos, &
              dpdt_eos, dpdr_eos, dedt_eos, dedr_eos, &
              dpdX_eos, dhdX_eos, &
              gam1_eos, cs_eos, s_eos, &
              dsdt_eos, dsdr_eos, &
              .false.)

     model(1,r_model_start) = rho0(r_model_start)
     model(2,r_model_start) = tempbar(r_model_start)

     pres(r_model_start) = p_eos

     do i=r_model_start+1,MAESTRO_npts_model-1

        ! use previous zone as initial guess for T, rho
        dens_zone = model(1,i-1)
        temp_zone = model(2,i-1)
        
        ! compute the gravitational acceleration on the interface between zones
        ! i-1 and i
        g_zone = -Gconst*M_enclosed(i-1)/(xznr(i-1)*xznr(i-1))

        !-----------------------------------------------------------------------
        ! iteration loop
        !-----------------------------------------------------------------------

        converged_hse = .FALSE.
        
        if (.not. hit_low_density_cutoff) then
           
           do iter = 1, MAX_ITER
              
              ! HSE differencing 
              p_want = pres(i-1) + dr*HALF*(dens_zone + model(1,i-1))*g_zone

              temp_zone = model(2,i-1)

              if (model(1,i-1) .lt. temp_fluff_density_cutoff) then
                 ! The MAESTRO initial model uses temp_zone = temp_fluff here
                 ! this helps smooth the density gradient at the edge of
                 ! the star and increases effectiveness of sponge
                 temp_zone = max(temp_fluff,0.9*temp_zone)
              end if
              
              ! (t, rho) -> (p)    
              temp_eos = temp_zone
              den_eos = dens_zone
              xn_eos(:) = xn(:)
              
              call eos(eos_input_rt, den_eos, temp_eos, &
                       xn_eos, &
                       p_eos, h_eos, e_eos, &
                       cv_eos, cp_eos, xne_eos, eta_eos, pele_eos, &
                       dpdt_eos, dpdr_eos, dedt_eos, dedr_eos, &
                       dpdX_eos, dhdX_eos, &
                       gam1_eos, cs_eos, s_eos, &
                       dsdt_eos, dsdr_eos, &
                       .false.)

              pres_zone = p_eos
              
              dpd = dpdr_eos
              drho = (p_want - pres_zone)/(dpd - HALF*dr*g_zone)

              dens_zone = max(0.9d0*dens_zone, &
                   min(dens_zone + drho, 1.1d0*dens_zone))
       
              if (abs(drho) < TOL*dens_zone) then
                 converged_hse = .TRUE.
                 exit
              endif

              if (dens_zone < low_density_cutoff) then
                 dens_zone = low_density_cutoff
                 temp_zone = temp_fluff
                 converged_hse = .TRUE.
                 hit_low_density_cutoff = .TRUE.
                 exit
              endif

           enddo
        
           if (.NOT. converged_hse) then
              print *, 'Error zone', i, ' did not converge in init_1d'
              print *, 'integrate up'
              print *, dens_zone, temp_zone
              print *, p_want
              print *, drho
              call bl_error('Error: HSE non-convergence')
           endif

           if (temp_zone < temp_fluff) then
              temp_zone = temp_fluff
           endif
           
        else
           dens_zone = low_density_cutoff
           temp_zone = temp_fluff
        endif

        ! call the EOS one more time for this zone and then go on to the next
        ! (t, rho) -> (p)    
        
        temp_eos = temp_zone
        den_eos = dens_zone
        xn_eos(:) = xn(:)
        
        call eos(eos_input_rt, den_eos, temp_eos, &
                 xn_eos, &
                 p_eos, h_eos, e_eos, &
                 cv_eos, cp_eos, xne_eos, eta_eos, pele_eos, &
                 dpdt_eos, dpdr_eos, dedt_eos, dedr_eos, &
                 dpdX_eos, dhdX_eos, &
                 gam1_eos, cs_eos, s_eos, &
                 dsdt_eos, dsdr_eos, &
                 .false.)

        pres(i) = p_eos
     
        ! update the thermodynamics in this zone
        model(1,i) = dens_zone
        model(2,i) = temp_zone

        M_enclosed(i) = M_enclosed(i-1) + &
             FOUR3RD*M_PI*(xznr(i) - xznl(i)) * &
             (xznr(i)**2 +xznl(i)*xznr(i) + xznl(i)**2)*model(1,i)

     enddo

   end subroutine ca_initdata_makemodel

   ! ::: -----------------------------------------------------------

   subroutine ca_initdata_overwrite(lo,hi, &
                                    state,state_l1,state_l2,state_l3, &
                                    state_h1,state_h2,state_h3, &
                                    model,model_size,MAESTRO_npts_model,dx,dr, &
                                    xlo,xhi,r_model_start)

     use meth_params_module, only : NVAR, URHO, UMX, UMY, UMZ, UEDEN, UEINT, UFS, UTEMP, UFX
     use network, only: nspec, naux
     use eos_module
     use probdata_module, only: center, bea_fuel, abar_fuel
     
     implicit none

     integer lo(3), hi(3), r_model_start
     integer model_size,MAESTRO_npts_model
     integer state_l1,state_l2,state_l3,state_h1,state_h2,state_h3
     double precision state(state_l1:state_h1,state_l2:state_h2,state_l3:state_h3,NVAR)
     double precision model(model_size,0:MAESTRO_npts_model-1)
     double precision dx(3), dr, xlo(3), xhi(3)
     double precision u(3)

     integer i,j,k,index,n

     double precision xcen,ycen,zcen,radius
     double precision temppres
     double precision :: xn(nspec+naux)

     do k=lo(3),hi(3)
        zcen = xlo(3) + dx(3)*(float(k-lo(3)) + 0.5d0) - center(3)
        do j=lo(2),hi(2)
           ycen = xlo(2) + dx(2)*(float(j-lo(2)) + 0.5d0) - center(2)
           do i=lo(1),hi(1)
              xcen = xlo(1) + dx(1)*(float(i-lo(1)) + 0.5d0) - center(1)

              ! compute associated radial array index
              radius = sqrt(xcen**2 + ycen**2 + zcen**2)
              index = int(radius/dr)

              if (index .gt. r_model_start) then

                 ! make sure model interpolation points are in bounds
                 if (index .eq. 0) then
                    index = 1
                 else if (index .ge. MAESTRO_npts_model-1) then
                    index = MAESTRO_npts_model-2
                 end if

                 ! we will "conserve velocity", not momentum, since
                 ! we are going to be dropping the density by many
                 ! orders of magnitude
                 u(1:3) = state(i,j,k,UMX:UMZ) / state(i,j,k,URHO)

                 ! compute rho
                 call quad_interp(radius, &
                                  (index-0.5d0)*dr,(index+0.5d0)*dr,(index+1.5d0)*dr, &
                                  state(i,j,k,URHO), &
                                  model(1,index-1),model(1,index),model(1,index+1))

                 ! compute T
                 call quad_interp(radius, &
                                  (index-0.5d0)*dr,(index+0.5d0)*dr,(index+1.5d0)*dr, &
                                  state(i,j,k,UTEMP), &
                                  model(2,index-1),model(2,index),model(2,index+1))

                 ! compute momentum
                 state(i,j,k,UMX:UMZ) = state(i,j,k,URHO)*u(1:3)

                 ! hard-code species and auxiliary quantities
                 xn(1) = 0.5d0
                 xn(2) = 0.5d0
                 xn(3) = 0.d0
                 xn(4) = 0.d0
                 xn(5) = 0.d0
                 xn(6) = 0.d0
                 xn(7) = 0.d0
                 xn(8) = 0.5d0
                 xn(9) = bea_fuel
                 xn(10)= abar_fuel

                 ! compute e
                 call eos_given_RTX(state(i,j,k,UEINT),temppres,state(i,j,k,URHO), &
                                    state(i,j,k,UTEMP),xn)

                 ! copy species into state
                 state(i,j,k,UFS:UFS+nspec-1) = xn(1      :nspec     )
                 state(i,j,k,UFX:UFX+naux -1) = xn(nspec+1:nspec+naux)

                 ! compute rho*e
                 state(i,j,k,UEINT) = state(i,j,k,URHO) * state(i,j,k,UEINT)

                 ! compute rho*E = rho*e + (1/2)*rho*|u|^2
                 state(i,j,k,UEDEN) = state(i,j,k,UEINT) &
                      + 0.5d0*(state(i,j,k,UMX)**2+state(i,j,k,UMY)**2+state(i,j,k,UMZ)**2) &
                      / state(i,j,k,URHO)

                 ! now that density has changed, recompute species and auxiliary variables
                 do n=1,nspec
                    state(i,j,k,UFS+n-1) = state(i,j,k,URHO) * state(i,j,k,UFS+n-1)
                 end do
                 do n=1,naux
                    state(i,j,k,UFX+n-1) = state(i,j,k,URHO) * state(i,j,k,UFX+n-1)
                 end do

              end if

           end do
        end do
     end do


   end subroutine ca_initdata_overwrite

   ! ::: -----------------------------------------------------------

  subroutine quad_interp(x,x0,x1,x2,y,y0,y1,y2)

    implicit none

    double precision x,x0,x1,x2,y,y0,y1,y2
    
    y = y0 + (y1-y0)/(x1-x0)*(x-x0) &
           + ((y2-y1)/(x2-x1)-(y1-y0)/(x1-x0))/(x2-x0)*(x-x0)*(x-x1)

    if (y .gt. max(y0,y1,y2)) y = max(y0,y1,y2)
    if (y .lt. min(y0,y1,y2)) y = min(y0,y1,y2)

  end subroutine quad_interp

