      subroutine ca_define_tau(lo,hi,  tau,  tau_l1,  tau_l2,  tau_l3, &
                                             tau_h1,  tau_h2,  tau_h3, &
                                     state,state_l1,state_l2,state_l3, &
                                           state_h1,state_h2,state_h3, &
                                      grav, grav_l1, grav_l2, grav_l3, &
                                            grav_h1, grav_h2, grav_h3, dx)

      use meth_params_module, only : NVAR, URHO, UTEMP, UFS
      use network
      use probdata_module, only : flame_thickening_factor, flamevel, flogrho, f_width, f_vel, &
           l_logrho, l_vel
      use burning_module
      use interpolate_module

      implicit none
      integer         , intent(in   ) :: lo(3),hi(3)
      integer         , intent(in   ) ::  tau_l1, tau_l2, tau_l3, tau_h1, tau_h2, tau_h3
      integer         , intent(in   ) :: state_l1,state_l2,state_l3,state_h1,state_h2,state_h3
      integer         , intent(in   ) :: grav_l1,grav_l2,grav_l3,grav_h1,grav_h2,grav_h3
      double precision, intent(inout) ::  tau(tau_l1:tau_h1,tau_l2:tau_h2,tau_l3:tau_h3)
      double precision, intent(in   ) :: state(state_l1:state_h1,state_l2:state_h2, &
                                               state_l3:state_h3,NVAR)
      double precision, intent(in   ) :: grav(grav_l1:grav_h1,grav_l2:grav_h2, &
                                              grav_l3:grav_h3,3)
      double precision, intent(in   ) :: dx(3)

      ! Local variables
      integer          :: i,j,k
      double precision :: temp,den,x12,rho_fuel
!     double precision :: g_eff
      double precision :: atwood,dummy
      double precision :: vel,flame_thickness
      double precision :: factor_w, factor_v
      double precision :: lvel

      integer,save          :: ic12
      logical, save :: firstCall = .true.

      !$OMP CRITICAL (define_tau_lock)
      if (firstCall) then
           if (.NOT. network_initialized) then
             call bl_error("ERROR in EOS: must initialize network first")
          endif

          ic12= network_species_index("carbon-12")
          
          firstCall = .false.
      endif
      !$OMP END CRITICAL (define_tau_lock)

      !$OMP PARALLEL DO PRIVATE(i,j,k,temp,den,x12,rho_fuel,dummy,atwood,factor_w,factor_v,flame_thickness,vel,lvel)
      do k = lo(3)-1,hi(3)+1
      do j = lo(2)-1,hi(2)+1
      do i = lo(1)-1,hi(1)+1

         temp = state(i,j,k,UTEMP)
         den = state(i,j,k,URHO)
         x12 = state(i,j,k,UFS+ic12-1) / den

         ! define rho_fuel from burning table
         call networkburn(log10(den),x12,rho_fuel,dummy,dummy,&
                         dummy,dummy,dummy,dummy,&
                         dummy,dummy,dummy,atwood) 

         ! define flame speed here.
         ! if you uncomment this, you need to fix the omp parallel do statement
!        g_eff = sqrt(grav(i,j,k,1)**2+grav(i,j,k,2)**2+grav(i,j,k,3)**2)
!        g_eff = atwood * g_eff

         factor_w = interpolate(rho_fuel, 17, flogrho,f_width)
         factor_v = interpolate(rho_fuel, 17, flogrho,f_vel)
         if( rho_fuel .lt. 6.9d0) then
                factor_w = 2.4d0
                factor_v = 2.9d0
         endif

         flame_thickness = factor_w*dble(flame_thickening_factor) * dx(1)

         ! limit flame speed to be maximum of flamevel and the minimum
         ! (laminar) flame speed from Timmes & Woosley (1992)
         lvel = interpolate(rho_fuel, 11, l_logrho, l_vel)
         vel = max(flamevel,lvel) * factor_v

         tau(i,j,k) = flame_thickness/vel

      enddo
      enddo
      enddo
      !$OMP END PARALLEL DO

      end subroutine ca_define_tau
